# -*- coding: utf-8 -*-
""" =============================  Tsukuba Express ============================
                                     for Thomas-san                           

csound/python process example for sound rendering control
Date: Created on Sonntag Oct  2015
Author: David Worrall <david.worrall@anu.edu.au
Version: 1.0
"""


""" ======================= RENDER ORCHESTRA ==============================="""
# <begin copy from orchestra definitions file>
# NB use ''' not """ for these string variables
sflowOrchestraOptions='''
sr = 48000
kr = 4800
ksmps = 10
nchnls = 2
0dbfs = 1.0
;gSaddr init "10.54.91.89"

;gSudpDemoRoom   init "10.54.91.89"  ; for udp comms to the demo room graphics machine 
;gSudpInet       init "10.54.91.151"; for udp comms to the Internet streaming graphics machine 

gSudpDemoRoom   init "127.0.0.1"; for udp comms to the Internet streaming graphics machine 
gSudpInet       init "127.0.0.1"; for udp comms to the Internet streaming graphics machine 

giudpDemoPort   init 4444
'''
sineInstr_01='''
instr 1
      instrNr       = p1
      istartTime    = p2
	idur	= p3
	iamp	= p4
	ifreq	= p5
      iattk = p6
      isus  = p7
      idky = p8
      irel = p9
 	kloc=	 p10	; location

	kbeta	= 0		; for ambi out

; === AUDIO SYNTH ===
    k1 madsr iattk, isus,irel, idky
    asig oscili iamp, ifreq, 1
                                ;    k1 madsr 0.02, 0.1, iamp*0.5, 0.1
                                ;    asig oscils iamp, ifreq, 1
    asig = asig * k1

; === UDP MESSAGE ===
;for UDP send - unique to this instr
    ScoreLabelSRC  strget p12   ; the label to be used in for the score display - passed via UDP
    ScoreLabelDST  strget p13   ; the label to be used in for the score display - passed via UDP
    ieventNr = p11          ; unique nr for each sonification event since beg of this instantiation
    ifreq1 = ifreq
    ifreq2 = ifreq

;for UDP send - clone for all instrs
   Stringy1  sprintf ",%i,%5.2f,%i,%5.2f,%5.2f,%5.2f", ieventNr, istartTime, instrNr,ifreq1, ifreq2, idur
   Stringy2  sprintf ",%s, %s", ScoreLabelSRC, ScoreLabelDST
   Stringy strcat Stringy1, Stringy2
        puts Stringy, 1     ; print the UDP message to the monitor

; === OUTPUT STAGE ===
    if (nchnls == 4) then    ; ambi out
        aw, ax, ay, az, ar, as, at, au, av bformenc1 asig, kloc, kbeta   ; generate Bfmat
        a1, a2, a3, a4 bformdec1 2, aw, ax, ay, az    ; decode Bfmat to 4chan speaker array
        outo a1, a2, a3, a4                           ; audio out

    elseif (nchnls == 2) then    ; stereo out
       aleft, aright pan2 asig, (kloc % 180) * 180    ; stereo out with panning (0=hardL, 1 = hardR)
        outs aleft, aright

    else out asig                                      ; play it in mono
    endif
endin
'''
pluckInstr_02='''
instr 2
      instrNr       = p1
      istartTime    = p2
    kloc	= p9 ; location
    kbeta	= 0   ; spread? for ambi out

    ifn  = 0 ; random
	idur	= p3
    ifreq	= p4
    imeth	= p6
    imethP1	= p7
    imethP2	= p8

; === AUDIO SYNTH ===
  asig pluck ifreq, p5, p5, ifn, imeth, imethP1, imethP2
  kres line 1, idur, 0  ; mainly so tail does not leave DC offset hanging.
  asig = asig * kres

; === UDP MESSAGE ===
;for UDP send - unique to this instr
    ieventNr = p10 ; unique event number for each sonification event since begining of this instantiation
    ScoreLabelSRC  strget p11 ; the label to be used in for the score display - passed via UDP
    ScoreLabelDST  strget p12 ; the label to be used in for the score display - passed via UDP
    ifreq1 = ifreq
    ifreq2 = ifreq

;for UDP send - clone for all instrs
   Stringy1  sprintf ",%i,%5.2f,%i,%5.2f,%5.2f,%5.2f", ieventNr, istartTime, instrNr,ifreq1, ifreq2, idur
   Stringy2  sprintf ",%s, %s", ScoreLabelSRC, ScoreLabelDST
   Stringy strcat Stringy1, Stringy2
        puts Stringy, 1     ; print the UDP message to the monitor

; === OUTPUT STAGE ===
    if (nchnls == 4) then   ; ambi out
        aw, ax, ay, az, ar, as, at, au, av bformenc1 asig, kloc, kbeta   ; generate Bfmat
        a1, a2, a3, a4 bformdec1 2, aw, ax, ay, az    ; decode Bfmat to 4chan speaker array
        outo a1, a2, a3, a4                           ; audio out

    elseif (nchnls == 2) then    ; stereo out
       aL, aR pan2 asig, (kloc % 180) * 180    ; stereo out with panning (0=hardL, 1 = hardR)
        outs aL, aR
    ; mono out - no UDP
   else out asig                                     ; mono out - no UDP
    endif
endin
'''
# <end copy from orchestra definitions file>

import csnd6
cs=csnd6.Csound()
cs.SetOption('-odac')
cs.SetOption('-b 4096')
csPerf =csnd6.CsoundPerformanceThread(cs)       # instantiate the RT perf thread
cs.ReadScore("f 0 z \n")                        # instantiate CS
# sound "font" for rendering"
cs.ReadScore("f 20 0 0 1 \"../../sounds/agogo1-H.wav\" 0 0 0 \n")
cs.ReadScore("f 21 0 0 1 \"../../sounds/agogo2-M.wav\" 0 0 0 \n")
# add more fonts here
# end soundFonts for rendering

cs.CompileOrc(sflowOrchestraOptions)         # SR, NrChans etc  # from sflowOrchestra.py

cs.Start()                                   # start CS synth
csPerf.Play()

cs.CompileOrc(sineInstr_01)     # compile an instrument- defined in sflowOrchestra.py 
cs.CompileOrc(pluckInstr_02)    # pluckInstr_02 is a variable name
# ....compile other instruments here....
cs.ReadScore("f 1 0 16384 10 1")
cs.ReadScore("f 2 0 17 0 0 1 1")
cs.ReadScore("f31 0 8 2 1 0.6 10 100 0.001 ; 1 rattle")
cs.ReadScore("f32 0 8 2 1 0.7 50 500 1000  ; 1 rubber")

# to suppress ALL terminal print output from csound
global TRACE 
TRACE = False
def passCallback():  pass
if not TRACE:
    cs.SetMessageCallback(passCallback())

# Generate code to send to RT csound, using a generic event generator
def makeCSevent (*args):
    """
    Makes a csound event and plays it in an already active Performance thread.
    Used to send strings of commands to csound. 
    """
    #Makes a csound event and plays it in an already active Performance thread
    eventStr = ' '.join(map(str, args))
    print eventStr
    cs.ReadScore (eventStr) #    emit it!

global OP_EVENT_COUNT
OP_EVENT_COUNT = 0
import time, random
def playAlap():
    global OP_EVENT_COUNT
    OP_EVENT_COUNT += 1
    dstScoreLabel = "destAddr"
    srcScoreLabel = "srcAddr"
    #global startTime
    startTime  = 0 
    amp = 1 
    azimuth = random.random()*180  # NB thus all notes in the nrNotes loop have same azimuth
    instrNr = 2 # pluck
    method = random.choice ([1, 3, ])  # 1 3 4 are possible
    mParam1 =  1 # random.choice ([0.8, 0.9, 1])
    mParam2 =  random.choice([1, 10])
    print "EVENT Nr: %i , Method: %i, P1: %2.1f, P2: %d"  % (OP_EVENT_COUNT, method, mParam1, mParam2)
    nrNotes = random.choice([3,5,8,13])
    fund = 55 * random.choice([7,8,9,10,11,12,13])
    startOffset = random.choice([x*0.167 for x in range(2)])
    for i in range(nrNotes):
        startOffset+=random.choice([0.167, 0.25, 0.25, 0.3])
        startOffset = startOffset +random.choice ([-0.01, 0, 0.01])
        startTime += startOffset *0.618
        freq = fund * random.choice([9/8.0, 8/9.0, 5/4.0, 4/5.0, 6/5.0, 5/6.0 ])
        durat = random.choice([1,2,3])
        makeCSevent ('i', 2, startTime, durat,  amp* 0.4, freq, method, mParam1,mParam2, azimuth, OP_EVENT_COUNT, srcScoreLabel, dstScoreLabel ) 
    time.sleep(startTime)

def playTsukubaExpress():
    global OP_EVENT_COUNT
    OP_EVENT_COUNT += 1
    dstScoreLabel = "destAddr"
    srcScoreLabel = "srcAddr"
    #global startTime
    startTime  = 0 
    amp = 1 
    azimuth = random.random()*180  # NB thus all notes in the nrNotes loop have same azimuth
    instrNr = 2 # pluck
    method = random.choice ([1, 3, ])  # 1 3 4 are possible
    mParam1 =  random.choice ([0.7, 0.8, 0.9, 1])
    mParam2 =  random.choice([1, 10])
    print "EVENT Nr: %i , Method: %i, P1: %2.1f, P2: %d"  % (OP_EVENT_COUNT, method, mParam1, mParam2)
#    makeCSevent ('i', 1, startTime, durat, amp, freq,    0.04, durat-0.05,  0, 0.01, azimuth, OP_EVENT_COUNT,dstScoreLabel , srcScoreLabel)
#    startTime += durat
    nrNotes = random.choice([3,5,8,13])
    fund = 110 * random.choice([1,2,3,4,5,6,7])
#    startOffset = random.choice([x*0.167 for x in range(2)])
    for i in range(nrNotes):
        startOffset = random.choice([0.167, 0.3])
        startOffset = startOffset +random.choice ([-0.01, 0, 0.01])
        startTime += startOffset *0.618
        freq = fund * random.choice([9/8.0, 8/9.0, 5/4.0, 4/5.0, 6/5.0, 5/6.0 ])
        durat = random.choice([1,2,3])
        makeCSevent ('i', 2, startTime, durat,  amp* 0.4, freq, method, mParam1,mParam2, azimuth, OP_EVENT_COUNT, srcScoreLabel, dstScoreLabel ) 
    time.sleep(startTime)

if not TRACE:
    print "\n++++++++++++++++++++++++++++++++++++++++++++++++++++++++"
    print "Ready to start the sonification from the user interface."
    print "++++++++++++++++++++++++++++++++++++++++++++++++++++++++"

while 1:
    if OP_EVENT_COUNT < 21:
        print "playAlap" ,
        playAlap()  # commenting out this line will cause no sound to be generated
    elif OP_EVENT_COUNT < 144:
        print "playTsukubaExpress" ,
        playTsukubaExpress()
    elif OP_EVENT_COUNT < 144+13:
        print "playTsukubaExpress" ,
        playAlap()  # commenting out this line will cause no sound to be generated
    elif OP_EVENT_COUNT > 144+13+1:
        print "FIN"
        break


if __name__ == "__main__":
    while 1:
        playTsukubaExpress() #doInner()

